#include "TArtCalibPID.h" 
#include "TArtPID.h" 
#include "TArtCore.h" 
#include "TArtPlastic.h"
#include "TArtFocalPlane.h"
#include "TArtFocalPlanePara.h"
#include "TArtRIPS.h"
#include "TArtRIPSPara.h"
#include "TArtTOF.h"
#include "TArtTOFPara.h"
#include "TArtBeam.h"
#include "TArtBeamPara.h"
#include "TArtBigRIPSParameters.h"

#include <TROOT.h>

//__________________________________________________________
TArtPID::TArtPID()
  :fReconstructed(false), fDetectorMapLoaded(false)
{
  TArtCore::Info(__FILE__,"Creating the PID objects...");
  setup = (TArtBigRIPSParameters*)gROOT->FindObject("BigRIPSParameters");
  Initialize();
}

//__________________________________________________________
TArtPID::TArtPID(TArtBigRIPSParameters * s)
  :fReconstructed(false), fDetectorMapLoaded(false), setup(s)
{
  TArtCore::Info(__FILE__,"Creating the PID objects...");
  Initialize();
}

//__________________________________________________________
TArtPID::TArtPID(TArtBigRIPSParameters * s, TArtCalibPID *calib)
  :fReconstructed(false), fDetectorMapLoaded(false), setup(s)
{
  TArtCore::Info(__FILE__,"Creating the PID objects...");
  Initialize(calib);
}

//__________________________________________________________
TArtPID::~TArtPID() {
  delete fpl_array;
  delete rips_array;
  delete tof_array;
  delete beam_array;
}

//__________________________________________________________
void TArtPID::Initialize() {
  fpl_array  = new TArtFocalPlaneArray(setup);
  rips_array = new TArtRIPSArray(setup);
  tof_array  = new TArtTOFArray(setup);
  beam_array = new TArtBeamArray(setup);
  return;
}

//__________________________________________________________
void TArtPID::Initialize(TArtCalibPID *calib) {
  Initialize();
  LoadDetectorMap(calib);
  return;
}

//__________________________________________________________
void TArtPID::ClearData() {
  fpl_array->ClearData();
  rips_array->ClearData();
  tof_array->ClearData();
  beam_array->ClearData();
  fReconstructed = false;
  return;
}

//
// function to map each detector to each objects
//__________________________________________________________
void TArtPID::LoadDetectorMap(TArtCalibPID *calib) {

  if(! fpl_array->IsPPACMapLoaded()) {
    TArtCore::Info(__FILE__,"Loading PPAC map for focal plane...");
    fpl_array->LoadPPACMap((TArtPPACArray*)calib->GetPPAC());
    TArtCore::Info(__FILE__,"Done: Loading PPAC map for focal plane...");
  }
  if(! fpl_array->IsPlasticMapLoaded()) {
    TArtCore::Info(__FILE__,"Loading Plastic map for focal plane...");
    fpl_array->LoadPlasticMap((TArtPlasticArray*)calib->GetPlastic()); 
    TArtCore::Info(__FILE__,"Done: Loading Plastic map for focal plane...");
  }
  if(! rips_array->IsFplMapLoaded()) {
    TArtCore::Info(__FILE__,"Loading focal plane map for rips...");
    rips_array->LoadFplMap(fpl_array); 
    TArtCore::Info(__FILE__,"Done: Loading focal plane map for rips...");
  }
  if(! tof_array->IsPlasticMapLoaded()) {
    TArtCore::Info(__FILE__,"Loading Plastic map for tof...");
    tof_array->LoadPlasticMap((TArtPlasticArray*)calib->GetPlastic()); 
    TArtCore::Info(__FILE__,"Done: Loading Plastic map for tof...");
  }
  if(! beam_array->IsMapLoaded()) {
    TArtCore::Info(__FILE__,"Loading map for beam reco...");
    beam_array->LoadMap(rips_array, tof_array, (TArtICArray*)calib->GetIC());
    TArtCore::Info(__FILE__,"Done: Loading map for beam reco...");
  }

  fDetectorMapLoaded = true;

}

//__________________________________________________________
void TArtPID::ReconstructData(TArtCalibPID *calib) {

  if(!fDetectorMapLoaded) LoadDetectorMap(calib);
  ReconstructData();

  return;
}

//__________________________________________________________
void TArtPID::ReconstructData() {

  if(!fDetectorMapLoaded) {
    TArtCore::Info(__FILE__,"Error:: load detector map for bigrips reconstruction...");
    return;
  }
  ClearData();

  // reconstruct beam position and angle by using PPAC data
  fpl_array->RecoBeamPos();

  // reconstruct delta of each rips by using fpl data
  rips_array->RecoDelta();

  // reconstruct tof by using plastic data
  tof_array->RecoTOF();

  // reconstruct beam 
  beam_array->RecoAoQZ();

  fReconstructed = true;

  return;

}


//__________________________________________________________
TArtFocalPlane * TArtPID::GetFocalPlane(int id) {
  TArtFocalPlane * fpl = NULL;
  for(int i=0;i<fpl_array->GetEntries();i++){
    fpl = (TArtFocalPlane *)fpl_array->At(i);
    if( fpl->GetID() == id ) return fpl;
  }
  return NULL;
}
//__________________________________________________________
TArtFocalPlane * TArtPID::GetFocalPlane(TArtFocalPlanePara * fp) {
  Int_t id = fp->GetID();
  return GetFocalPlane(id);
}
//__________________________________________________________
Int_t TArtPID::GetNumFocalPlane() {
  return fpl_array->GetEntries();
}
//__________________________________________________________
TArtRIPS * TArtPID::GetRIPS(int id) {
  TArtRIPS * r = NULL;
  for(int i=0;i<rips_array->GetEntries();i++){
    r = (TArtRIPS *)rips_array->At(i);
    if( r->GetID() == id ) return r;
  }
  return NULL;
}
//__________________________________________________________
TArtRIPS * TArtPID::GetRIPS(TArtRIPSPara * rp) {
  Int_t id = rp->GetID();
  return GetRIPS(id);
}
//__________________________________________________________
Int_t TArtPID::GetNumRIPS() {
  return rips_array->GetEntries();
}
//__________________________________________________________
TArtTOF * TArtPID::GetTOF(int id) {
  TArtTOF * t = NULL;
  for(int i=0;i<tof_array->GetEntries();i++){
    t = (TArtTOF *)tof_array->At(i);
    if( t->GetID() == id ) return t;
  }
  return NULL;
}
//__________________________________________________________
TArtTOF * TArtPID::GetTOF(TArtTOFPara * tp) {
  Int_t id = tp->GetID();
  return GetTOF(id);
}
//__________________________________________________________
Int_t TArtPID::GetNumTOF() {
  return tof_array->GetEntries();
}
//__________________________________________________________
TArtBeam * TArtPID::GetBeam(int id) {
  TArtBeam * b = NULL;
  for(int i=0;i<beam_array->GetEntries();i++){
    b = (TArtBeam *)beam_array->At(i);
    if( b->GetID() == id ) return b;
  }
  return NULL;
}
//__________________________________________________________
TArtBeam * TArtPID::GetBeam(TArtBeamPara * bp) {
  Int_t id = bp->GetID();
  return GetBeam(id);
}
//__________________________________________________________
Int_t TArtPID::GetNumBeam() {
  return beam_array->GetEntries();
}
